# urlfilter

[urlfilter] (https://dev.iopsys.eu/iopsys/urlfilter)

## Introduction

urlfilter blocks list of urls for the list of mac addresses passed to it. *urlfilter* uses the libnetfilter-queue along with libnetlink.
APIs to achieve the requirement.

## Project Components

Project consists of following components:

- Application itself written in C programming language

## Build Instructions

`urlfilter` is written using C programming language and depends on a number of components found in `OpenWrt` for building and running.
 - libnetfilter-queue
 - libnfnetlink
 - iptables-mod-nfqueue

## Pre-requisites

Before running the application you need to add a rule in the iptables to forward the http, https and dns query/response packets to NFQUEUE.
NFQUEUE is an iptables and ip6tables target which delegate the decision on packets to a userspace software.

- iptables -I FORWARD 1 -p tcp --dport 80 -j NFQUEUE --queue-num 0
- iptables -I FORWARD 2 -p tcp --dport 443 -j NFQUEUE --queue-num 0
- iptables -I INPUT 1 -p udp --dport 53 -j NFQUEUE --queue-num 0
- iptables -I INPUT 2 -p udp --sport 53 -j NFQUEUE --queue-num 0

In userspace, urlfilter daemon uses libnetfilter-queue to connect to queue 0 (the default one) and get the messages from kernel.
It then issues a verdict on the packet after checking the  mac address and url.

## Sample UCI configuration
URL filtering daemon requires a configuration file. The configuration file is an uci file `/etc/config/urlfilter`. Sample configuration file is provided below.


````bash
# cat /etc/config/parentalcontrol

config globals 'globals'
	option enable '1'

config profile 'profile_1'
	option name 'kids'
	list host 'aa:00:aa:35:01:01'
	list host 'D_wan_53_1'
	option bedtime_enable '1'
	list internet_access_schedule 'schedule_1'
	option internet_access_policy 'Allow'
	option internet_access_enable '1'

config profile_urlfilter 'profile_1_profile_urlfilter_1'
	option dm_parent 'profile_1'
	list filter_text 'facebook.com'
	list profile_urlfilter_schedule 'schedule_1'
	option profile_urlbundle 'b_kids'
	option enable '1'

config urlbundle 'urlbundle_1'
	option name 'b_kids'
	list custom_url 'youtube.com'

config profile_bedtime_schedule 'profile_1_profile_bedtime_schedule_1'
	option dm_parent 'profile_1'
	option start_time '21:00'
	option end_time '08:00'
	list day 'Saturday'
	list day 'Sunday'

# cat /etc/config/schedules

config global 'global'
	option enable '1'

config schedule 'schedule_1'
	option enable '1'
	option duration '28800'
	list day 'Monday'
	list day 'Tuesday'
	list day 'Wednesday'
	list day 'Thursday'
	list day 'Friday'
	option start_time '13:00'
````

